// Michael D. (mike0815) 09-09-2017
// Update: 15-09-2017

// Temparaturgesteuerte (PWM) Lueftersteuerung für 2 Lufter an 2 Ports mit LM35 TempSensor, 2x16 LCD-Display
// (Temparaturanzeige u. Lüfterdrehzahl in Prozentangabe, 3 Status LEDs
// Arduino UNO Board 16MHz. Portiert für ATMega8 8MHz interner Takt

// Luefter-Steuerung mit LM35 Sensor. Sensor mißt mit interner Referenz, (ATMega328) von 1,1V statt 5V(Boardspannung).
// Berechnung der internen Referenz: 1.1/   1024=1.0742mV  1°C=10mV/ 1.0742=9.31 
// Beim ATMega8 ist die interne Ref: 2.56V/ 1024=2.5000mV  1°C=10mV/ 2.5000=4.00

// 7%-100%, 14 Stufen fuer 2 getrennte Luefter(je 150mA) an 2 Ports(2x Analog Output mit 1k an Basis mit z.B. 2N2222) 

// Luefter je 80mm, 150mA:0%-(off)   7% 14% 21% 28% 35% 42% 50% 57% 64% 71% 78% 85% 92% 100%
// 0%- 7% = LED(Gruen)  = on. (Status) 
// Ab 16%-100% LED(Rot) = off LED(Gruen) = off
// 16%-100% = LED(Blau) = on.
// Temp1Min (z.B."36"(36°C), starten die Luefter),
// Temp1Min (z.B."50"(50°C), 100% max. Drehzahl (14,0V)

// M328/88/48 4830 Bytes ohne Serial output. 2736 Bytes ohne Seriel und LCD-Display Ausgabe.
// Fusebits: Low:0xFF(16MHz-Crystal), High:0xDE, Extendet:0xFD(0x05), (Lock:0xCF)
// lfuse:0xE2 (8MHz-Internal) Change in boards.txt
// Syntax: ""uno.build.f_cpu=8000000Luno.build.core=arduinouno.build.variant=standard""
// Der ArduinoUNO muss mit 16MHz Programmiert werden, sonst gibt es ein fail!
// ------ATMega8 Fusebits:  *** Low: 0xEE ***(16MHz)/(0xE4(8MHz int)) High:D9   *** Lock:0F--------
// ------ATMega8 Arduino NG *** Low: 0xDF ***(16MHz)/(0xE4(8MHz int)) High:0xCA *** Lock:0xFF /(CF)--------for programming from Sketch
// ------Sometimes when read the Flash from M8, it's not possible to transfer the "Sketch"! Write the LowFuse(0xDF)again, solve the Problem!
// ------After the Sketch is transfered or the complete HEX-File(inkl.Bootloader) is burned, LowFuse must be write to 0xE4 (to 8MHz internal Clock) 
// ------Compile Sketch for 8MHz(internal), "board.txt" must be edit from 16MHz to 8MHz!!! 

#include <LiquidCrystal.h>
LiquidCrystal lcd(7,6,5,4,3,2);
//LiquidCrystal            rs, rw(GND), enable, d4,   d5,   d6,   d7
//LCD-Ports:              (PD7) (GND)   (PD6)  (PD5) (PD4) (PD3) (PD2)

int LM351 = A0;         // output pin   (PC0) of LM35
//int LM352 = A1; 
int fan1 = 10;          // pin  fan1    (PB2)  B-->1k  C-->Fan1(-)  E-->(GND) (5-15V)
int fan2 = 11;          // pin  fan2    (PB3)  B-->1k  C-->Fan2(-)  E-->(GND) (5-15V)
int ledLOW = 8;         // led pin      (PB0) (low) (green) when Fans are off (Status)
int ledHIGH = 9;        // led pin      (PB1) (high) (red)   when Fans are on
int ledALERT = 12;      // led pin      (PB4) (ALERT)(Blue)  when over MaxTemp
float temp1;            // 4x Digit display   (for testing)
float vref = 2.560; //Atmega8-Ref.= 2.56V
//int reading;
//int temp1;            // 2x Digit display   (Default) when enabled then "float temp1" must be disabled!!!
//int temp2;
int temp1Min = 30.00;   // temperature when fan starts
int temp1Max = 52.00;   // maximum temperature fan reached 100%
//int temp2Min = 25;    // temperature when fan starts
//int temp2Max = 70;    // maximum temperature fan reached 100%
int fan1Speed;
int fan2Speed;
int fan1LCD;
int fan2LCD;

void setup() {
  pinMode(fan1, OUTPUT);
  pinMode(fan2, OUTPUT);
  pinMode(ledLOW, OUTPUT);    //(green) for Fan off      (Status)
  pinMode(ledHIGH, OUTPUT);   //(red)   for Fan starting (temp1Min Status)
  pinMode(ledALERT, OUTPUT);  //(orange)  when Overtemp    (Status)
  pinMode(LM351, INPUT);      // LM35 SensPin
//  pinMode(LM352, INPUT);
analogReference(INTERNAL);    // for ATMega8 2,56V internal Reference
//Serial.begin(9600);
  lcd.begin(16,2);
//lcd.begin(20,4);    
}
 
void loop() {  

  temp1 = readTemp();     // get temperature
//reading = analogRead(LM351);
   if(temp1 < temp1Min) { // if temp is lower than minimum temp 
      fan1Speed = 0; // fan stop
      fan2Speed = 0; // fan stop 
      digitalWrite(fan1, LOW);
      digitalWrite(fan2, LOW);


  
   } 
   if((temp1  >= temp1Min) && (temp1 <= temp1Max)) {         // if temperature is higher than minimum temp 
      fan1Speed = map(temp1, temp1Min, temp1Max, 32, 255);   // the actual speed of fan1
      fan2Speed = map(temp1, temp1Min, temp1Max, 32, 255);   // the actual speed of fan2 
      fan1LCD = map(temp1, temp1Min, temp1Max, 0, 100);      // speed of fan to display on LCD
      fan2LCD = map(temp1, temp1Min, temp1Max, 0, 100);      // speed of fan to display on LCD 
      analogWrite(fan1, fan1Speed);                          // spin the fan1 at the fanSpeed  
      analogWrite(fan2, fan2Speed);                          // spin the fan2 at the fanSpeed  

   }
 // ledlow MD---------
   if(temp1  > temp1Min) {                                   // if temp is lower than tempMin
     digitalWrite(ledLOW, HIGH);                             // turn on LED (green) if lower than tempMin 
   } else {                    
     digitalWrite(ledLOW, LOW);                              // else turn of LED (green) if higher than tempMin
   
   } 
   if(temp1  >= temp1Min) {                                  // if temp is higher or equal than tempMin 
     digitalWrite(ledHIGH, LOW);                             // turn of LED (Orange)
   } else {                    
     digitalWrite(ledHIGH, HIGH);                            // turn on LED (Orange)from tempMin to tempMax
   }
   
 // ledALERT OverTEMP--------
   if(temp1  > temp1Max) {                                   // if temp is higher than tempMax
     digitalWrite(ledALERT, LOW);                            // turn on LED (Red)when overTemp
   } else {                    
     digitalWrite(ledALERT, HIGH);                           // turn of LED (Red)
   }   

   
/*   {
  digitalWrite(ledALERT, HIGH);                              // turn the LED on (HIGH is the voltage level)
  delay(500);                                                // wait for a second
  digitalWrite(ledALERT, LOW);                               // turn the LED off by making the voltage LOW
  delay(500);                                                // wait for a second
}*/

  // ledlow end MD------ 
   lcd.setCursor(0,0);                  // move cursor to first line(top left)
   lcd.print("Temp:");
   lcd.setCursor(6,0);                  // move cursor to first line(top position 6)
   lcd.print(temp1);                    // display the temperature
   lcd.setCursor(12,0);                 // move cursor to first line(top position 9)
   lcd.print("c");
//lcd.setCursor(12,0);                  // move cursor to first line(top position 10)
//lcd.print("LM35");
   
  lcd.setCursor(0,1);                   // move cursor to next line (bottom left)
   lcd.print("F1:");                    // Display Fan1
      lcd.print(fan1LCD);               // Display fanspeed
lcd.setCursor(6,1);                     // move cursor to next line (bottom position 6)
   lcd.print("%");                      // Display %
lcd.setCursor(9,1);                     // move cursor to next line (bottom position 9)
   lcd.print("F2:");                    // Display Fan2
   lcd.print(fan2LCD);                  // display fanspeed
lcd.setCursor(15,1);                    // move cursor to next line (bottom position 15)
   lcd.print("%");
   delay(200);
   lcd.clear();   
}
 
int readTemp() {  // get temperature
  temp1 = analogRead(LM351);
//reading = analogRead(LM351);  
//  temp2 = analogRead(LM352);
//  return temp1 * 0.48828125;
// -----------M328 internal Ref. 1.1/ 1024=1.0742mV  1°C=10mV/ 1.0742=9.31  --------------MD
// -----------M8  internal Ref. 2.56/ 1024=4.0000mV  1°C=10mV/ 2.5000=4.00  --------------MD
//   return temp1 /9.31; //Ref. 1.1V Atmega328
   return temp1 /3.8; //Ref. 2.56V Atmega8 (Ref.= 2.601V = 3,93)
}
